!   EZFIO is an automatic generator of I/O libraries
!   Copyright (C) 2009 Anthony SCEMAMA, CNRS
!
!   This program is free software; you can redistribute it and/or modify
!   it under the terms of the GNU General Public License as published by
!   the Free Software Foundation; either version 2 of the License, or
!   (at your option) any later version.
!
!   This program is distributed in the hope that it will be useful,
!   but WITHOUT ANY WARRANTY; without even the implied warranty of
!   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!   GNU General Public License for more details.
!
!   You should have received a copy of the GNU General Public License along
!   with this program; if not, write to the Free Software Foundation, Inc.,
!   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
!
!   Anthony Scemama
!   LCPQ - IRSAMC - CNRS
!   Universite Paul Sabatier
!   118, route de Narbonne      
!   31062 Toulouse Cedex 4      
!   scemama@irsamc.ups-tlse.fr

logical function ezfio_exists(path)
  implicit none
  character*(*) :: path
  inquire(file=trim(path)//'/.version',exist=ezfio_exists)
  if (ezfio_exists) then
   open(unit=libezfio_iunit,file=trim(path)//'/.version')
   character*(32) :: V
   read(libezfio_iunit,*) V
   close(libezfio_iunit)
   integer :: char_to_version
   if (char_to_version(V) > char_to_version(libezfio_version)) then
     call ezfio_error(irp_here, "This file was generated with version "//trim(V)//&
     " but the current installed version is "//trim(libezfio_version)//".")
   endif
  endif
end function

subroutine ezfio_mkdir(path)
  implicit none
  character*(*) :: path
  logical  :: ezfio_exists
  if (libezfio_read_only) then
    call ezfio_error(irp_here,'Read-only file.')
  endif
  if (.not.ezfio_exists(path)) then
    call c_mkdir(trim(path))
    open(unit=libezfio_iunit,file=trim(path)//'/.version')
    write(libezfio_iunit,'(A)') libezfio_version
    close(libezfio_iunit)
  endif

end subroutine 

BEGIN_SHELL [ /usr/bin/python ]
from f_types import format, t_short


template = """
subroutine ezfio_read_%(type_short)s(dir,fil,dat)
  implicit none
  character*(*), intent(in)  :: dir, fil
  %(type)s, intent(out) :: dat
  character*(1024) :: l_filename
  l_filename=trim(dir)//'/'//fil
  open(unit=libezfio_iunit,file=l_filename,form='FORMATTED', &
  action='READ',err=9)
  read(libezfio_iunit,%(fmt)s,end=9,err=9) dat
  close(libezfio_iunit)
  return
9 continue
  call ezfio_error(irp_here,'Attribute '//trim(dir)//'/'//trim(fil)//' is not set')
end

subroutine ezfio_write_%(type_short)s(dir,fil,dat)
  implicit none
  character*(*), intent(in)  :: dir, fil
  %(type)s, intent(in) :: dat
  character*(1024) :: l_filename(2)
  if (libezfio_read_only) then
    call ezfio_error(irp_here,'Read-only file.')
  endif
  l_filename(1)=trim(dir)//'/.'//fil
  l_filename(2)=trim(dir)//'/'//fil
  open(unit=libezfio_iunit,file=l_filename(1),form='FORMATTED',action='WRITE')
  write(libezfio_iunit,%(fmt)s) dat
  close(libezfio_iunit)
  call system( 'mv '//trim(l_filename(1))//' '//trim(l_filename(2)) )
end

subroutine ezfio_read_array_%(type_short)s(dir,fil,rank,dims,dim_max,dat)
  implicit none
  character*(*), intent(in)  :: dir, fil
  integer             :: rank
  integer             :: dims(rank)
  integer             :: dim_max
  %(type)s            :: dat(dim_max)
  integer :: err
  character*(128) :: buffer
  character*(1024) :: l_filename
  l_filename=trim(dir)//'/'//fil//'.gz'

  err = 0
  call libezfio_openz(trim(l_filename),'rb',err)
  if (err == 0) then
    integer :: rank_read
    integer :: dims_read(rank), i

    call libezfio_readz(buffer,err)
    read(buffer,'(I3)') rank_read
    if (rank_read /= rank) then
      call ezfio_error(irp_here,'Rank of data '//trim(l_filename)//&
      ' different from array.')
    endif

    call libezfio_readz(buffer,err)
    if (err /= 0) then
      call ezfio_error(irp_here,'Error reading data in '//trim(l_filename)//&
      '.')
    endif
    read(buffer,'(30(I20,X))') dims_read(1:rank)
    do i=1,rank
      if (dims_read(i) /= dims(i)) then
        call ezfio_error(irp_here,'Dimensions of data '//trim(l_filename)//&
        ' different from array.')
      endif
    enddo
   
    do i=1,dim_max
     call libezfio_readz(buffer,err)
     if (err /= 0) then
      call ezfio_error(irp_here,'Error reading data in '//trim(l_filename)//&
      '.')
     endif
     read(buffer,%(fmt)s) dat(i)
    enddo
    call libezfio_closez()
    return
  else
    call ezfio_error(irp_here,'Attribute '//trim(l_filename)//' is not set')
  endif
end

subroutine ezfio_write_array_%(type_short)s(dir,fil,rank,dims,dim_max,dat)
  implicit none
  character*(*), intent(in)  :: dir, fil
  integer, intent(in) :: rank
  integer, intent(in) :: dims(rank)
  integer, intent(in) :: dim_max
  %(type)s, intent(in) :: dat(dim_max)
  integer :: err
  character*(128) :: buffer
  character*(1024) :: l_filename(2)
  if (libezfio_read_only) then
    call ezfio_error(irp_here,'Read-only file.')
  endif
  l_filename(1)=trim(dir)//'/.'//fil//'.gz'
  l_filename(2)=trim(dir)//'/'//fil//'.gz'

  err = 0
  call libezfio_openz(trim(l_filename(1)),'wb',err)
  if (err == 0) then
    write(buffer,'(I3)') rank
    call libezfio_writez(trim(buffer))
    write(buffer,'(30(I20,X))') dims(1:rank)
    call libezfio_writez(trim(buffer))

    integer :: i
    do i=1,dim_max
     write(buffer,%(fmt)s) dat(i)
     call libezfio_writez(trim(buffer))
    enddo
    call libezfio_closez()
  endif
  call system( 'mv '//trim(l_filename(1))//' '//trim(l_filename(2)) )
end
"""

template_no_logical = """
integer function n_count_%(type_short)s(array,isize,val)
  %(type)s, intent(in) :: array(*)
  integer, intent(in)  :: isize
  %(type)s, intent(in) :: val

  integer :: i
  n_count_%(type_short)s = 0
  do i=1,isize
   if (array(i) == val) then
     n_count_%(type_short)s = n_count_%(type_short)s +1
   endif
  enddo

end function
! Build Python functions
"""
for t in format.keys():
  print template%{ 'type_short' : t_short(t), 'type' : t, 'fmt':format[t][0] }
  if t != "logical":
    print template_no_logical%{ 'type_short' : t_short(t), 'type' : t, 'fmt':format[t][0] }

template_py = """
  def read_%(type_short)s(self,dir,fil):
    conv = get_conv("%(type_short)s")
    l_filename=dir.strip()+'/'+fil
    try:
      file = open(l_filename,"r")
    except IOError:
      self.error('read_%(type_short)s',\
         'Attribute '+dir.strip()+'/'+fil+' is not set')
    dat = file.readline().strip()
    try:
      dat = conv(dat)
    except SyntaxError:
      pass
    file.close()
    self.access()
    return dat

  def write_%(type_short)s(self,dir,fil,dat):
    if self.read_only:
      self.error('Read-only file.')
    conv = get_conv("%(type_short)s")
    l_filename= [ dir.strip()+'/.'+fil ]
    l_filename += [ dir.strip()+'/'+fil ]
    dat = conv(dat)
    file = open(l_filename[0],'w')
    print >>file,'%(fmt)s'%%(dat,)
    file.close()
    os.rename(l_filename[0],l_filename[1])
    self.access()

  def read_array_%(type_short)s(self,dir,fil,rank,dims,dim_max):
    l_filename=dir.strip()+'/'+fil+'.gz'
    conv = get_conv("%(type_short)s")
    try:
      file = GzipFile(filename=l_filename,mode='rb')
      lines = map(lambda x: x.strip(), file.read().splitlines())
      rank_read = int(lines[0])
      assert (rank_read == rank)
     
      dims_read = map(int,lines[1].split())
     
      for i,j in zip(dims_read,dims):
        assert i == j
      
      dat = []

      for data in lines[2:dim_max+2]:
        try:
          data = conv(data)
        except:
          pass
        dat.append(data)
      file.close()
      self.access()
      return reshape(dat,dims)
        
    except IOError:
      self.error('read_array_%(type_short)s', 'Attribute '+l_filename+' is not set')

  def write_array_%(type_short)s(self,dir,fil,rank,dims,dim_max,dat):
    if self.read_only:
      self.error('Read-only file.')
    l_filename  = [ dir.strip()+'/'+fil+'.gz.tmp' ]
    l_filename += [ dir.strip()+'/'+fil+'.gz' ]
    try:
      file = StringIO.StringIO()
      file.write("%%3d\\n"%%(rank,))
      for d in dims:
        file.write("%%20d "%%(d,))
      file.write("\\n")

      dat = flatten(dat)
      for i in xrange(dim_max):
        file.write("%(fmt)s\\n"%%(dat[i],))
      file.flush()
      buffer = file.getvalue()
      file.close()
      file = GzipFile(filename=l_filename[0],mode='wb')
      file.write(buffer)
      file.close()
      os.rename(l_filename[0],l_filename[1])
      self.access()
    except:
      self.error("write_array_%(type_short)s",
        "Unable to write "+l_filename[0])
"""

file_py = open("libezfio_util-gen.py","w")
for t in format.keys():
  print >>file_py, template_py%{ 'type_short' : t_short(t), 'type' : t, 'fmt':format[t][1] }

import os
command = "'echo %s > '//libezfio_filename//'/ezfio/last_library'"
cwd = os.getcwd()
cwd = cwd.split('src')[:-1]
cwd = '/'.join(cwd)
print >>file_py, """
  LIBRARY = "%s"
"""%cwd

file_py.close()
END_SHELL

BEGIN_PROVIDER [ integer, libezfio_buffer_rank ]
 BEGIN_DOC  
! Rank of the buffer ready for reading
 END_DOC
 libezfio_buffer_rank = -1
END_PROVIDER

subroutine ezfio_open_write_buffer(dir,fil,rank)
  implicit none
  character*(*),intent(in)    :: dir
  character*(*),intent(in)    :: fil
  integer,intent(in)          :: rank
  character*(1024) :: l_filename
  if (libezfio_read_only) then
    call ezfio_error(irp_here,'Read-only file.')
  endif
  l_filename=trim(dir)//'/'//fil//'.gz'

  if (libezfio_buffer_rank /= -1) then
    call ezfio_error(irp_here,'Another buffered file is already open.')
  endif

  libezfio_buffer_rank = rank
  if (libezfio_buffer_rank <= 0) then
    call ezfio_error(irp_here,'In file '//trim(l_filename)//': rank <= 0.')
  endif
  TOUCH libezfio_buffer_rank

  integer :: err
  call libezfio_openz(trim(l_filename),'wb',err)
  if (err /= 0) then
    call ezfio_error(irp_here,'Unable to open buffered file '//trim(l_filename)//'.')
  endif

  character*(4) :: buffer
  write(buffer,'(I2)') rank
  call libezfio_writez(trim(buffer))

end

subroutine ezfio_open_read_buffer(dir,fil,rank)
  implicit none
  character*(*),intent(in)    :: dir
  character*(*),intent(in)    :: fil
  integer,intent(in)         :: rank
  character*(1024) :: l_filename
  l_filename=trim(dir)//'/'//fil//'.gz'

  if (libezfio_buffer_rank /= -1) then
    call ezfio_error(irp_here,'Another buffered file is already open.')
  endif

  integer :: err
  call libezfio_openz(trim(l_filename),'rb',err)
  if (err /= 0) then
    print *,  err, l_filename
    call ezfio_error(irp_here,'Unable to open buffered file '//trim(l_filename)//'.')
  endif

  character*(8) :: buffer
  call libezfio_readz(buffer,err)
  if (err /= 0) then
    print *,  err, l_filename
    call ezfio_error(irp_here,'Unable to read buffered file '//trim(l_filename)//'.')
  endif

  read(buffer,'(I2)') libezfio_buffer_rank
  if (libezfio_buffer_rank /= rank) then
    call ezfio_error(irp_here,'In file '//trim(l_filename)//': Rank is not correct')
  endif
  TOUCH libezfio_buffer_rank

end

subroutine ezfio_close_buffer()
  ASSERT (libezfio_buffer_rank > 0)
  call libezfio_closez()
  FREE libezfio_buffer_rank
end

subroutine ezfio_read_buffer(indices,values,isize)
  implicit none

  integer                       :: isize
  integer, intent(out)          :: indices(*)
  double precision, intent(out) :: values(isize)

  character*(8192) :: buffer
  integer         :: i, j, eof

  if (libezfio_buffer_rank == -1) then
    call ezfio_error(irp_here,'No buffered file is open.')
  endif

  do i=1,isize
   call libezfio_readz(buffer,eof)
   if (eof /= 0) then
     isize=i-1
     return
   endif
   read(buffer,*) (indices((i-1)*libezfio_buffer_rank+j), j=1,libezfio_buffer_rank), values(i)
  enddo

end

subroutine ezfio_write_buffer(indices,values,isize)
  implicit none

  integer, intent(in)          :: isize
  integer, intent(in)          :: indices(*)
  double precision, intent(in) :: values(isize)

  integer, parameter  :: LENBUF=16384
  character*(LENBUF) :: buffer
  character*(80)   :: cformat
  integer          :: i, j, k, num, imax, l1, l2

  if (libezfio_read_only) then
    call ezfio_error(irp_here,'Read-only file.')
  endif
  if (libezfio_buffer_rank == -1) then
    call ezfio_error(irp_here,'No buffered file is open.')
  endif
  
  num = LENBUF/(5*libezfio_buffer_rank+25) 
  write(cformat,*) '(',num,'(',libezfio_buffer_rank,'(I4,X),E24.15,A1))'
  do k=1,isize,num
    imax = min(k+num-1,isize)
    l2 = (5*libezfio_buffer_rank+25)*num+1
    write(buffer,cformat) ((indices((i-1)*libezfio_buffer_rank+j), j=1,libezfio_buffer_rank), &
          values(i),'\\', i=k,imax)
    l1 = l2
    do while (buffer(l2:l2) /= '\\')
     l2 = l2-1
     ASSERT (l2>0)
    enddo
    buffer(l2:l2) = ' '
    call libezfio_writez(buffer)
  enddo

end

