/*  EZFIO is an automatic generator of I/O libraries
    Copyright (C) 2009 Anthony SCEMAMA, CNRS
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Anthony Scemama
    LCPQ - IRSAMC - CNRS
    Universite Paul Sabatier
    118, route de Narbonne      
    31062 Toulouse Cedex 4      
    scemama@irsamc.ups-tlse.fr
*/


#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <errno.h>
#include "zlib.h"
#define SET_BINARY_MODE(file)

#define CHUNK 262144

static gzFile file = NULL;

void libezfio_openz_(const char* filename_in, const char* mode_in, int* error, long size_filename, long size_mode)
{
  char filename[size_filename+1];
  char mode[size_mode+1];
  int i;
  
  for (i=0 ; i<size_filename ; i++)
    filename[i] = filename_in[i];
  filename[i] = 0;

  for (i=0 ; i<size_mode ; i++)
    mode[i] = mode_in[i];
  mode[i] = 0;

  file = gzopen (filename, mode);
  if (file == NULL)
    *error = 1;
  else
    *error = 0;
}


void libezfio_closez_()
{
  gzclose(file);
  file = NULL;
}


void libezfio_readz_(char* buffer_out, int* eof, long size_buffer)
{
  char buffer[size_buffer+1];
  int i;
  if (gzgets(file,buffer,size_buffer))
  {
    *eof = 0;
    i=0;
    while (i<size_buffer && buffer[i] != '\n' && buffer[i] != 0)
    {
      buffer_out[i] = buffer[i];
      i++;
    }
    while (i<size_buffer)
    {
      buffer_out[i] = ' ';
      i++;
    }
  }
  else
    *eof = 1;
}

void libezfio_writez_(const char* buffer_in, long size_buffer_in)
{
  char buffer[size_buffer_in+2];
  int i;
  int size_buffer;
  
  size_buffer = size_buffer_in;

  for (i=0 ; i<size_buffer ; i++)
  {
    if (buffer_in[i] != '\\')
      buffer[i] = buffer_in[i];
    else
      buffer[i] = '\n';
  }

  while (buffer[--i] == ' ');
  buffer[i+1] = '\n';
  buffer[i+2] = 0;

  gzputs(file,buffer);
}

void c_mkdir_(const char* buffer_in, long size_buffer_in)
{
  char buffer[size_buffer_in+1];
  int i;
  int size_buffer;

  size_buffer = size_buffer_in;

  for (i=0 ; i<size_buffer ; i++)
  {
    buffer[i] = buffer_in[i];
  }
  buffer[size_buffer] = 0;
  if (mkdir(buffer, 0777) == -1)
    if (errno != EEXIST)
      printf("Unable to create directory %s", buffer);
}

#ifdef TEST
int main()
{
  int error;
  char line[80];
  libezfio_openz_("test.gz", "wb7", &error, 8L, 3L);
  strcpy(line,"Test line");
  libezfio_writez_(line,strlen(line)+1);
  strcpy(line,"Test line 2");
  libezfio_writez_(line,strlen(line)+1);
  libezfio_closez_();
  libezfio_openz_("test.gz", "rb", &error, 8L, 3L);
  libezfio_readz_(line,&error,80);
  printf(":%s:%d\n",line,error);
  libezfio_readz_(line,&error,80);
  printf(":%s:%d\n",line,error);
  libezfio_readz_(line,&error,80);
  printf(":%s:%d\n",line,error);
  libezfio_closez_();
  return 0;
}
#endif
